-- Black Widow arcade hardware implemented in an FPGA
-- (C) 2012 Jeroen Domburg (jeroen AT spritesmods.com)
-- 
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
-- 
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
-- 
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.

-- Silly little first-order delta-sigma converter

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;



entity deltasigma is
	generic (
		width: integer :=4
   );
	Port ( inval : in  STD_LOGIC_VECTOR (width-1 downto 0);
           output : out  STD_LOGIC;
           clk : in  STD_LOGIC;
           reset : in  STD_LOGIC);
end deltasigma;

architecture Behavioral of deltasigma is
signal reg: STD_LOGIC_VECTOR(width+1 downto 0);
signal reg_d: STD_LOGIC_VECTOR(width+1 downto 0);
signal ddcout: STD_LOGIC_VECTOR(width+1 downto 0);

begin
ds: process(clk, reset)
begin
	if reset='1' then
		reg<=(others => '0');
		output<='0';
	elsif rising_edge(clk) then
		reg<=reg_d;
		output<=reg(width);
	end if;
end process;
ddcout(width+1 downto width)<="00";
ddcout(width-1 downto 0)<=(others=>'1') when reg(width)='1' else (others => '0');
reg_d<=(("00"&inval)-ddcout)+reg;
end Behavioral;

