-- PSX controller interface,

-- Black Widow arcade hardware implemented in an FPGA
-- (C) 2012 Jeroen Domburg (jeroen AT spritesmods.com)
-- 
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
-- 
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
-- 
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity psxcontroller is
    Port ( psxdata : in  STD_LOGIC;
           psxcmd : out  STD_LOGIC;
           psxatt : out  STD_LOGIC;
           psxclk : out  STD_LOGIC;
           psxack : in  STD_LOGIC;
           buttons : out  STD_LOGIC_VECTOR (15 downto 0);
           clk : in  STD_LOGIC;
			  clken : in STD_LOGIC;
			  reset_l : in STD_LOGIC);
end psxcontroller;

	
architecture psxcontroller_arch of psxcontroller is
	type statetype is (IDLE, START, CMDGETID, GETID, DATAHDR, DATA1, DATA2, ENDING);
	signal state: statetype;
	signal byteIn: STD_LOGIC_VECTOR (7 downto 0);
	signal byteOut: STD_LOGIC_VECTOR (7 downto 0);
	signal byteOutSh: STD_LOGIC_VECTOR (7 downto 0);
	signal shiftPos: STD_LOGIC_VECTOR (3 downto 0);
	signal clkdiv: STD_LOGIC_VECTOR (7 downto 0);
	signal myButtons: STD_LOGIC_VECTOR (15 downto 0);
	signal gotByte: STD_LOGIC;
	signal sendByte: STD_LOGIC;
	signal clkOut: STD_LOGIC;
	signal clkMainEn: STD_LOGIC;
begin
   -- Clock div
	process(reset_l, clk)
	begin
		if reset_l = '0' then
			clkdiv<="00000000";
			clkMainEn<='0';
		elsif clk'EVENT and clk = '1' and clken = '1' then
			clkdiv<=clkdiv+"1";
			if clkdiv = "00000000" then
				clkMainEn<='1';
			else
				clkMainEn<='0';
			end if;
		end if;
	end process;

	-- Main routine
	process(reset_l, clk)
	begin
		if reset_l = '0' then
			state <= IDLE;
			myButtons <= "0000000000000000";
		elsif clk'EVENT and clk = '1' and clkMainEn = '1' and clken='1' then
			if gotByte = '1' and sendByte='0' then
				if state=IDLE then
					psxatt <= '0';
					state <= START;
				elsif state=START then
					byteOut<="00000001";
					sendByte<='1';
					state <= CMDGETID;
				elsif state=CMDGETID then
					-- byteIn is a nonexisting response to cmd 1
					byteOut<="01000010";
					sendByte<='1';
					state <= GETID;
				elsif state=GETID then
					-- byteIn now is controller type. /care
					byteOut<="00000000";
					sendByte<='1';
					state <= DATAHDR;
				elsif state=DATAHDR then
					-- byteIn now is 0x5A
					byteOut<="00000000";
					sendByte<='1';
					state <= DATA1;
				elsif state=DATA1 then
					-- byteIn now is SELECT X X START UP RIGHT DOWN LEFT
					myButtons(7 downto 0)<=not byteIn;
--					byteOut<=myButtons;
					sendByte<='1';
					state <= DATA2;
				elsif state=DATA2 then
					-- byteIn now is L2 R2 L1 R1 TRIANGLE CIRCLE CROSS SQUARE
					myButtons(15 downto 8)<=not byteIn;
					state <= ENDING;
				elsif state=ENDING then
					psxatt <= '1';
					state <= IDLE;
				end if;
			else
				sendByte<='0';
			end if;
		end if;
	end process;

	-- Shift register
	process(reset_l, clk)
	begin
		if reset_l = '0' then
			shiftPos <= "1010";
			gotByte <= '1';
			clkOut <= '1';
		elsif clk'EVENT and clk = '1' and clkMainEn='1' and clken='1' then
			if shiftPos<"1000" then
				if clkOut = '0' then
					-- rising edge -> sample
					for i in 0 to 6 loop
						byteIn(i) <= byteIn(i+1);
					end loop;
					byteIn(7) <= psxdata;
					if shiftPos /= "0111" then 
						clkOut <= '1';
					end if;
					shiftPos <= shiftPos + 1;
				else
					-- falling edge - output
					psxcmd <= byteOutSh(0);
					for i in 0 to 6 loop
						byteOutSh(i) <= byteOutSh(i+1);
					end loop;
					clkOut <= '0';
				end if;
			elsif shiftPos = "1000" then
				clkOut<='1';
--				if psxack = '0' then
					shiftPos <= shiftPos + 1;
--				end if;
			elsif shiftPos = "1001" then
--				if psxack = '1' then
					shiftPos <= shiftPos + 1;
--				end if;
				gotByte <= '1';
			else
				if sendByte='1' then
					shiftPos <= "0000";
					gotByte <= '0';
					byteOutSh <= byteOut;
				end if;
			end if;
		end if;
	end process;
	
	psxclk<=clkOut;
	buttons<=myButtons;
end psxcontroller_arch;

